// 002 u_c_generate_dataset
// 01 oct 2006

// -- (C) John COLIBRI 2006
// -- documentation: http://www.jcolibri.com

(*$r+*)

unit u_c_generate_dataset;
  interface
    uses Classes, IbQuery, u_c_basic_object
        , u_c_generator_2;

    type c_generate_dataset= class(c_generator)
                               m_table_name, m_first_field_name: String;
                               m_class_name, m_constructor_name: String;
                               m_field_underscore: String;

                               m_c_ibquery: tIbQuery;

                               _m_c_event_declaration_generator: c_generator;
                               _m_c_event_assignment_generator: c_generator;
                               _m_c_field_event_assignment_generator: c_generator;
                               _m_c_event_handler_generator: c_generator;

                               Constructor create_generate_dataset(p_table_name,
                                   p_first_field_name, p_class_name, p_constructor_name,
                                   p_field_underscore: String;
                                   p_c_ibquery: tIbQuery);

                                 procedure _generate_events;
                                 procedure _generate_header;
                                 procedure _generate_class;
                                 procedure _generate_interface;
                                 procedure _generate_methods;
                                 procedure _generate_end;

                               procedure generate_dataset;
                             end; // c_generate_dataset

  implementation
    uses SysUtils, Db
        , u_strings
        , u_c_display
        // -- circular, to get the "options" (could use parameters)
        , u_business_objects_generator
        , u_c_generate_sql_queries
        ;

    // -- c_generate_dataset

    Constructor c_generate_dataset.create_generate_dataset(p_table_name,
        p_first_field_name, p_class_name, p_constructor_name, p_field_underscore: String;
        p_c_ibquery: tIbQuery);
      begin
        Inherited create_generator(p_table_name);

        m_table_name:= p_table_name;
        m_first_field_name:= p_first_field_name;
        m_class_name:= p_class_name;
        m_constructor_name:= p_constructor_name;
        m_field_underscore:= p_field_underscore;

        m_c_ibquery:= p_c_ibquery;
      end; // create_generate_dataset

    procedure c_generate_dataset._generate_events;

      function f_get_first_field_name: String;
        begin
          with m_c_ibquery do
          begin
            Result:= FieldDefs[0].Name;
          end;
        end; // f_get_first_field_name

      var l_event_count: Integer;
          l_field_event_count: Integer;

      procedure generate_event(p_do_generate: Boolean; p_method_name, p_event_name: String);
        begin
          if p_do_generate
            then begin
                with _m_c_event_declaration_generator do
                begin
                  if l_event_count= 0
                    then add_line('');
                  add_line('  procedure '+ p_method_name+ '(p_c_dataset: tDataset);');
                end; // with _m_c_event_declaration_generator

                with _m_c_event_assignment_generator do
                begin
                  if l_event_count= 0
                    then begin
                        add_line('');
                        add_line('// -- hook the events');
                      end;
                  add_line(p_event_name+ ':= '+ p_method_name+ ';');
                end; // with _m_c_event_assignment_generator

                with _m_c_event_handler_generator do
                begin
                  add_line('');
                  add_line_indent('procedure '+ m_class_name+ '.'+ p_method_name+ '(p_c_dataset: tDataset);');
                  add_line('begin');
                  if (p_method_name= 'before_post') and Form1.first_field_gen_.Checked
                    then begin
                        add_line('  // -- get next key');
                        add_line('  if State= dsInsert');
                        add_line('    then '+ m_first_field_name+ m_field_underscore+ '.Value:= dm_database.f_get_next_key('''+ m_table_name+ '_generator'');');
                      end;
                  add_line_unindent('end; // '+ p_method_name);
                end; // _m_c_event_handler_generator
              end;
          Inc(l_event_count);
        end; // generate_event

      procedure generate_field_event(p_do_generate: Boolean; p_field_name, p_method_name, p_event_name: String);
        var l_field_method_name: String;
        begin
          if p_do_generate
            then begin
                l_field_method_name:= p_field_name+ '_'+ p_method_name;
                with _m_c_event_declaration_generator do
                begin
                  if l_field_event_count= 0
                    then add_line('');
                  add_line('  procedure '+ l_field_method_name+ '(p_c_field: tField);');
                end; // with _m_c_event_declaration_generator

                with _m_c_field_event_assignment_generator do
                begin
                  if l_field_event_count= 0
                    then begin
                        add_line('');
                        add_line('// -- hook the field_events');
                      end;
                  add_line(p_field_name+ m_field_underscore+ '.'+ p_event_name+ ':= '+ l_field_method_name+ ';');
                end; // with _m_c_field_event_assignment_generator

                with _m_c_event_handler_generator do
                begin
                  add_line('');
                  add_line_indent('procedure '+ m_class_name+ '.'+ l_field_method_name+ '(p_c_field: tField);');
                  add_line('begin');
                  add_line_unindent('end; // '+ l_field_method_name);
                end; // _m_c_event_handler_generator
              end;
          Inc(l_field_event_count);
        end; // generate_field_event

      var l_validate_field_name: String;

      begin // _generate_events
        with Form1 do
        begin
          // -- check the events
          l_event_count:= 0;

          // generate_event(True, 'after_open', 'AfterOpen');
          generate_event(before_edit_.Checked, 'before_edit', 'BeforeEdit');
          generate_event(before_post_.Checked, 'before_post', 'BeforePost');

          // -- generate a sample field event
          if first_field_validate_.Checked
            then begin
                l_validate_field_name:= LowerCase(f_get_first_field_name);
                l_field_event_count:= 0;
                generate_field_event(True, l_validate_field_name, 'validate', 'OnValidate');
              end;
        end; // with Form1
      end; // _generate_events

    procedure c_generate_dataset._generate_header;
      begin
        add_line('// 001 u_'+ m_class_name);
        add_line('// 01 oct 2006');
        add_line('');
        add_line('(*$r+*)');
        add_line('');

        add_line_indent('unit u_'+ m_class_name+ ';');

        add_line_indent('interface');
        add_line('uses Classes, Db, IbDatabase, IBCustomDataSet;');
      end; // _generate_header

    procedure c_generate_dataset._generate_class;

      procedure generate_field_declaration;

        procedure add_field(p_field_name, p_field_type: String);
          var l_field_declaration: String;
          begin
            l_field_declaration:= '  '+ p_field_name+ m_field_underscore+ ': '+ p_field_type+ ';';
            add_line(l_field_declaration);
          end; // add_field

        var l_field_def_index: Integer;

        begin // generate_field_declaration
          with m_c_ibquery do
          begin
            with FieldDefs do
              for l_field_def_index:= 0 to Count- 1 do
                with FieldDefs[l_field_def_index] do
                begin
                  case DataType of
                    // ftUnknown,
                    ftString : add_field(Name, 'tStringField');

                    // , ftSmallint,
                    ftInteger : add_field(Name, 'tIntegerField');
                    // 4 , ftWord, ftBoolean,
                    ftFloat : add_field(Name, 'tFloatField');
                    // 7          , ftCurrency, ftBCD, ftDate, ftTime,
                    ftDateTime : add_field(Name, 'tDateTimeField');
                    // 12 , ftBytes, ftVarBytes, ftAutoInc, ftBlob, ftMemo, ftGraphic, ftFmtMemo, ftParadoxOle, ftDBaseOle, ftTypedBinary, ftCursor, ftFixedChar, ftWideString, ftLargeint, ftADT, ftArray, ftReference, ftDataSet, ftOraBlob, ftOraClob, ftVariant, ftInterface, ftIDispatch, ftGuid, ftTimeStamp, ftFMTBcd

                    else display_bug_stop('missing_field '+ IntToStr(Ord(DataType)));
                  end; // case
                end; // with FieldDefs, for l_field_def_index
          end; // with m_c_ibquery, Sql
        end; // generate_field_declaration

      procedure generate_routines;
        begin
          // -- add the methods after the fields
          add_line('');

          add_line('  constructor '+ m_constructor_name+ '(p_c_owner: tComponent;');
          add_line('      p_c_ib_database: tIbDatabase; p_select_sql: String);');
          add_line('  procedure open_dataset();');

          add_line('');
          // -- always include, because of master-detail open/close
          add_line('  procedure after_open(p_c_dataset: tDataset);');

          // -- insert the event declarations
          append_generator_lines_indented(_m_c_event_declaration_generator, m_indentation);
        end; // generate_routines

      var l_class_indentation: Integer;

      begin // _generate_class
        add_line('');
        l_class_indentation:= Length('type '+ m_class_name)+ 2;
        add_line('type '+ m_class_name+ '= Class(tIbDataSet)');
        Inc(m_indentation, l_class_indentation);

        generate_field_declaration;

        generate_routines;

        m_indentation:= Length('type')+ l_class_indentation;
        add_line('end; // '+ m_class_name);
      end; // _generate_class

    procedure c_generate_dataset._generate_interface;
      begin
        m_indentation:= 2;
        add_line('');
        add_line_indent('implementation');
        add_line('uses u_dm_database;');
      end; // _generate_interface

    procedure c_generate_dataset._generate_methods;

      procedure generate_constructor_body;

        procedure generate_change_requests;

          procedure add_query(p_request: String; p_c_request_value: tStringList);
            var l_list_index: Integer;
                l_result: String;
            begin
              l_result:= '';
              with p_c_request_value do
                for l_list_index:= 0 to Count- 1 do
                begin
                  l_result:= l_result+ ''''+ Strings[l_list_index]+ '''';
                  if l_list_index< Count- 1
                    then l_result:= l_result+ Chr(13)+ chr(10)+ f_spaces(Self.m_indentation+ 4)+ '+ ';
                end; // with p_c_request_value, for l_list_index

              add_line(p_request+ l_result+ ';');
            end; // add_query

          begin // generate_change_requests
            with c_generate_sql_queries.create_generate_sql_queries(m_table_name, m_first_field_name) do
            begin
              generate_sql_queries(m_c_ibquery);

              add_query('ModifySql.Text:= ', m_c_update_sql);
              add_query('InsertSql.Text:= ', m_c_insert_sql);
              add_query('DeleteSql.Text:= ', m_c_delete_sql);
              Free;
            end; // with c_generate_sql_queries
          end; // generate_change_requests

        begin // generate_constructor_body
          // -- constructor body
          add_line_indent('Constructor '+ m_class_name+ '.'+ m_constructor_name+ '(p_c_owner: tComponent;');
          add_line('  p_c_ib_database: tIbDatabase; p_select_sql: String);');
          add_line_indent('begin');
          add_line('Inherited Create(p_c_owner);');
          add_line('');
          add_line('DataBase:= p_c_ib_database;');
          add_line('SelectSql.Text:= p_select_sql;');

          if not Form1.read_only_.Checked
            then begin
                generate_change_requests;
              end;

          Dec(m_indentation, 2);

          // -- link the event initialization
          // --   always hook OnAfterOpen event first
          add_line('');
          add_line('  AfterOpen:= after_open;');
          append_generator_lines_indented(_m_c_event_assignment_generator, m_indentation+ 2);

          add_line_unindent('end; // '+ m_constructor_name);
        end; // generate_constructor_body

      procedure generate_open_body;
        begin
          add_line('');
          add_line_indent('procedure '+ m_class_name+ '.open_dataset();');
          add_line_indent('begin');
          add_line('Open;');
          Dec(m_indentation, 2);
          add_line_unindent('end; // open_dataset');
        end; // generate_open_body

      procedure generate_after_open_body;

        procedure generate_field_ref_initialization;

          procedure add_field(p_field_name, p_field_type: String);
            begin
              add_line('    if Name= '''+ p_field_name+ '''');
              add_line('      then '+ p_field_name+ m_field_underscore+ ':= Fields[l_field_index] as '+ p_field_type+ ' else');
            end; // add_field

          var l_field_def_index: Integer;

          begin // generate_field_ref_initialization
            with m_c_ibquery do
            begin
              with FieldDefs do
                for l_field_def_index:= 0 to Count- 1 do
                  with FieldDefs[l_field_def_index] do
                  begin
                    case DataType of
                      // ftUnknown,
                      ftString : add_field(Name, 'tStringField');

                      // , ftSmallint,
                      ftInteger : add_field(Name, 'tIntegerField');
                      // 4 , ftWord, ftBoolean,
                      ftFloat : add_field(Name, 'tFloatField');
                      // 7          , ftCurrency, ftBCD, ftDate, ftTime,
                      ftDateTime : add_field(Name, 'tDateTimeField');
                      // 12 , ftBytes, ftVarBytes, ftAutoInc, ftBlob, ftMemo, ftGraphic, ftFmtMemo, ftParadoxOle, ftDBaseOle, ftTypedBinary, ftCursor, ftFixedChar, ftWideString, ftLargeint, ftADT, ftArray, ftReference, ftDataSet, ftOraBlob, ftOraClob, ftVariant, ftInterface, ftIDispatch, ftGuid, ftTimeStamp, ftFMTBcd

                      else display_bug_stop('missing_field '+ IntToStr(Ord(DataType)));
                    end; // case
                  end; // with FieldDefs, for l_field_def_index
            end; // with m_c_ibquery
          end; // generate_field_ref_initialization

        begin // generate_open_body
          // -- start the opening / field validation loop
          add_line('');
          add_line_indent('procedure '+ m_class_name+ '.after_open(p_c_dataset: tDataset);');
          add_line('var l_field_index: Integer;');
          add_line_indent('begin');

          add_line('// -- check which columns are included, and link them to our attributes');
          add_line('for l_field_index:= 0 to FieldCount- 1 do');
          add_line('  with FieldDefs[l_field_index] do');

          generate_field_ref_initialization;

          // -- link the field event initialization
          append_generator_lines_indented(_m_c_field_event_assignment_generator, m_indentation+ 2);

          Dec(m_indentation, 2);
          add_line_unindent('end; // after_open_dataset');
        end; // generate_open_body

      begin // _generate_methods
        add_line('');
        add_line('// -- '+ m_class_name);
        add_line('');

        generate_constructor_body;
        generate_open_body;
        generate_after_open_body;

        // -- append the event bodies
        append_generator_lines_indented(_m_c_event_handler_generator, m_indentation);
      end; // _generate_methods

    procedure c_generate_dataset._generate_end;
      begin
        m_indentation:= 2;
        add_line('');
        add_line('end.');
      end; // _generate_end

    procedure c_generate_dataset.generate_dataset;
      begin
        // -- first generate all events code to centralize which events will be used
        _m_c_event_declaration_generator:= c_generator.create_generator(m_table_name);
        _m_c_event_assignment_generator:= c_generator.create_generator(m_table_name);
        _m_c_field_event_assignment_generator:= c_generator.create_generator(m_table_name);
        _m_c_event_handler_generator:= c_generator.create_generator(m_table_name);

        _generate_events;

        _generate_header;
        _generate_class;
        _generate_interface;
        _generate_methods;
        _generate_end;

        _m_c_event_declaration_generator.Free;
        _m_c_event_assignment_generator.Free;
        _m_c_field_event_assignment_generator.Free;
        _m_c_event_handler_generator.Free;
      end; // generate_dataset

    begin // u_c_generate_dataset
    end. // u_c_generate_dataset


