// 002 u_c_generate_module
// 30 sep 2006

// -- (C) John COLIBRI 2006
// -- documentation: http://www.jcolibri.com

(*$r+*)

unit u_c_generate_module;
  interface
    uses u_c_basic_object
        , u_c_generator_2;

    type c_generate_module= class(c_generator)
                              m_table_name: String; // , m_first_field_name: String;
                              m_dataset_class_name, m_dataset_constructor_name: String;
                              m_module_suffix: String;

                              _m_datamodule_class_name, _m_datamodule_constructor_name: String;
                              _m_dataset_name, _m_datasource_name: String;

                              Constructor create_generate_module(p_table_name,
                                  p_dataset_class_name, p_dataset_constructor_name,
                                  p_module_suffix: String);

                                procedure _generate_header;
                                procedure _generate_class;
                                procedure _generate_interface;
                                procedure _generate_routine_body;
                                procedure _generate_end;
                              procedure generate_module;
                            end; // c_generate_module

  implementation
    uses SysUtils, u_c_display;

    // -- c_generate_module

    Constructor c_generate_module.create_generate_module(p_table_name,
        p_dataset_class_name, p_dataset_constructor_name,
        p_module_suffix: String);
      begin
        Inherited create_generator(p_table_name);
        m_table_name:= p_table_name;

        m_dataset_class_name:= p_dataset_class_name;
        m_dataset_constructor_name:= p_dataset_constructor_name;
        m_module_suffix:= p_module_suffix;
      end; // create_generate_module

    procedure c_generate_module._generate_header;
      begin
        _m_datamodule_class_name:= 'c_'+ m_table_name+ m_module_suffix;
        _m_dataset_name:= '_'+ m_table_name;
        _m_datasource_name:= '_'+ m_table_name+ '_datasource';

        add_line('// 001 u_'+ _m_datamodule_class_name);
        add_line('// 01 oct 2006');
        add_line('');
        add_line('(*$r+*)');
        add_line('');

        add_line_indent('unit u_'+ _m_datamodule_class_name+ ';');

        add_line_indent('interface');
        add_line('uses Classes, Db, IBCustomDataSet, u_'+ m_dataset_class_name+ ';');
      end; // _generate_header

    procedure c_generate_module._generate_class;
      var l_datamodule_class_indentation: Integer;

      procedure generate_attribute_declaration;
        begin
          add_line('  '+ _m_dataset_name+ ': '+ m_dataset_class_name+ ';');
          add_line('  '+ _m_datasource_name+ ': tDataSource;');
        end; // generate_attribute_declaration

      procedure generate_routine_declaration;
        begin
          // -- add the methods after the fields
          add_line('');
          _m_datamodule_constructor_name:= 'create_'+ m_table_name+ m_module_suffix;

          add_line('  constructor '+ _m_datamodule_constructor_name+ '(p_c_owner: tComponent;');
          add_line('    p_select_sql: String);');
          add_line('  procedure _open_dataset();');
          add_line('  procedure _close_dataset();');
        end; // generate_routine_declaration

      begin // _generate_class
        add_line('');
        l_datamodule_class_indentation:= Length('type '+ _m_datamodule_class_name)+ 2;
        add_line('type '+ _m_datamodule_class_name+ '= Class(tComponent)');
        Inc(m_indentation, l_datamodule_class_indentation);

        generate_attribute_declaration;

        generate_routine_declaration;

        m_indentation:= Length('type')+ l_datamodule_class_indentation;
        add_line('end; // '+ _m_datamodule_class_name);
      end; // _generate_class

    procedure c_generate_module._generate_interface;
      begin
        m_indentation:= 2;
        add_line('');
        add_line_indent('implementation');
        add_line('  uses u_dm_database;');
      end; // _generate_interface

    procedure c_generate_module._generate_routine_body;

      procedure generate_constructor_body;
        begin
          // -- start the constructor body
          add_line_indent('Constructor '+ _m_datamodule_class_name+ '.'+ _m_datamodule_constructor_name+ '(p_c_owner: tComponent;');
          add_line('  p_select_sql: String);');
          add_line_indent('begin');
          add_line('Inherited Create(p_c_owner);');

          add_line('');
          // -- create the ibDataSet
          add_line(_m_dataset_name+ ':= '+ m_dataset_class_name+ '.'+ m_dataset_constructor_name+ '(Self, dm_database.IbDataBase1, p_select_sql);');

          add_line('');
          add_line('// -- create and link a datasource');
          add_line(_m_datasource_name+ ':= tDataSource.Create(Self);');
          add_line(_m_datasource_name+ '.DataSet:= '+ _m_dataset_name+ ';');
          Dec(m_indentation, 2);
          add_line_unindent('end; // '+ _m_datamodule_constructor_name);
        end; // generate_constructor_body

      procedure generate_open_dataset_body;
        begin
          add_line('');
          add_line_indent('procedure '+ _m_datamodule_class_name+ '._open_dataset();');
          add_line_indent('begin');
          add_line(_m_dataset_name+ '.open_dataset();');
          Dec(m_indentation, 2);
          add_line_unindent('end; // _open_dataset');
        end; // generate_open_dataset_body

      procedure generate_close_dataset_body;
        begin
          add_line('');
          add_line_indent('procedure '+ _m_datamodule_class_name+ '._close_dataset();');
          add_line_indent('begin');
          add_line(_m_dataset_name+ '.Close();');
          Dec(m_indentation, 2);
          add_line_unindent('end; // _close_dataset');
        end; // generate_close_dataset_body

      begin // _generate_routine_body
        add_line('');
        add_line('// -- '+ _m_datamodule_class_name);
        add_line('');

        generate_constructor_body;
        generate_open_dataset_body;
        generate_close_dataset_body;
      end; // _generate_routine_body

    procedure c_generate_module._generate_end;
      begin
        m_indentation:= 2;
        add_line('');
        add_line('end.');
      end; // _generate_end

    procedure c_generate_module.generate_module;
      begin
        _generate_header;
        _generate_class;
        _generate_interface;
        _generate_routine_body;
        _generate_end;
      end; // generate_module

    begin // u_c_generate_module
    end. // u_c_generate_module


