// 002 u_c_generator_2
// 30 sep 2006

// -- (C) John COLIBRI 2006
// -- documentation: http://www.jcolibri.com

// -- the CURRENT indentation will be prepended during "add_line"
// --  => can change the indentation before generating the line
(*$r+*)

unit u_c_generator_2;
  interface
    uses Classes, u_c_basic_object;

    type c_generator= class(c_basic_object)
                        m_c_result_list: tStringList;
                        m_current_line: String;
                        m_indentation: Integer;

                        Constructor create_generator(p_name: String);

                        function f_c_self: c_generator;
                        procedure clear_generator;

                        procedure add_text(p_text: String);
                        procedure add_line(p_text: String);
                        procedure add_new_line;

                        procedure indent_add_line(p_text: String);
                        procedure unindent_add_line(p_text: String);
                        procedure add_line_indent(p_text: String);
                        procedure add_line_unindent(p_text: String);

                        procedure append_generator_lines(p_c_generator: c_generator);
                        procedure append_generator_lines_indented(p_c_generator: c_generator; p_indentation: Integer);

                        procedure save_to_file(p_full_file_name: String);

                        Destructor Destroy; Override;
                      end; // c_generator

  implementation
    uses SysUtils, u_strings, u_c_display, u_stringlist;

    // -- c_generator

    Constructor c_generator.create_generator(p_name: String);
      begin
        Inherited create_basic_object(p_name);

        m_c_result_list:= tStringList.Create;
      end; // create_generator

    function c_generator.f_c_self: c_generator;
      begin
        Result:= Self;
      end; // f_c_self

    procedure c_generator.clear_generator;
      begin
        m_c_result_list.Clear;
        m_current_line:= '';
        m_indentation:= 0;
      end; // clear_generator

    procedure c_generator.add_text(p_text: String);
      begin
        m_current_line:= m_current_line+ p_text;
      end; // add_text

    procedure c_generator.add_line(p_text: String);
      begin
        m_c_result_list.Add(f_spaces(m_indentation)+ m_current_line+ p_text);
        m_current_line:= '';
      end; // add_line

    procedure c_generator.indent_add_line(p_text: String);
      begin
        Inc(m_indentation, 2);
        add_line(p_text);
      end; // indent_add_line

    procedure c_generator.unindent_add_line(p_text: String);
      begin
        Dec(m_indentation, 2);
        add_line(p_text);
      end; // unindent_add_line

    procedure c_generator.add_new_line;
      begin
        add_line('');
      end; // add_new_line

    procedure c_generator.add_line_indent(p_text: String);
      begin
        add_line(p_text);
        Inc(m_indentation, 2);
      end; // add_line_indent

    procedure c_generator.add_line_unindent(p_text: String);
      begin
        add_line(p_text);
        Dec(m_indentation, 2);
      end; // add_line_unindent

    procedure c_generator.append_generator_lines(p_c_generator: c_generator);
      begin
        append_second_stringlist(m_c_result_list, p_c_generator.m_c_result_list);
      end; // append_generator_lines

    procedure c_generator.append_generator_lines_indented(p_c_generator: c_generator; p_indentation: Integer);
      var l_index: Integer;
          l_indentation: String;
      begin
        l_indentation:= f_spaces(p_indentation);
        for l_index:= 0 to p_c_generator.m_c_result_list.Count- 1 do
          m_c_result_list.Add(l_indentation+ p_c_generator.m_c_result_list[l_index]);
      end; // append_generator_lines_indented

    procedure c_generator.save_to_file(p_full_file_name: String);
      begin
        m_c_result_list.SaveToFile(p_full_file_name);
      end; // save_to_file

    Destructor c_generator.Destroy;
      begin
        m_c_result_list.Free;
        Inherited;
      end; // Destroy

    begin // u_c_generator
    end. // u_c_generator
